/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.repository');


/**
 * Create the panel
 */
org.ametys.repository.AdminTab = function ()
{
	this._panel = new Ext.FormPanel ({
		id: 'descriptors-panel',
		layout: 'form',	
		region: 'center',
        border: false,    	
        forceLayout: true,
		//split: true,
		autoScroll:true,		
	    layoutConfig: {
        	trackLabels: true
		},
		labelAlign: 'left',
		labelWidth: 250,

		cls:'admin-panel',
		
		items: []
	});
	
	/**
	 * Returns the global panel
	 */
	return new Ext.Panel ({
		title: "<i18n:text i18n:key="ADMIN_TAB_TITLE"/>",
		id: 'admin-tab',
		layout: 'border',
		region: 'center',
		listeners: { 
			'beforeshow': function() {
								org.ametys.repository.AdminTab._requestData();
							}
		},
		items: [ 
		        this._panel
	         ]
	});
	
}


/**
 * Requests descriptors of the repository
 */
org.ametys.repository.AdminTab._requestData = function()
{	
    var serverMessage = new org.ametys.servercomm.ServerMessage(null, workspaceURI+'/repository/descriptors',
			{}, 
			org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, 
			null, 
			this, 
			null);
	
	var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);

	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="ERROR_MSG_GET_DESCRIPTORS"/>", response, "org.ametys.cms.tool.user.Edit"))
	{
		return;
	}

	var descriptorsNodes = response.selectSingleNode("repository").selectNodes("descriptor");

	var panel = Ext.getCmp('descriptors-panel');
	panel.removeAll();	
	
	if (descriptorsNodes == null)
	{
		return;
	}	
	
	// for each descriptor, creation of a label
	for (var i=0; i &lt; descriptorsNodes.length; i++)
	{
		var descriptor = descriptorsNodes[i];
		
		// get property attributes and value(s)
		var key = descriptor.getAttribute("key");
		var value = descriptor.getAttribute("value");

		panel.add(new Ext.form.Label({
				fieldLabel: key,
				text: value,
				width: 250})
		);
	} 
	
	panel.doLayout();

}
