/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.repository');

org.ametys.repository.PropertiesFormPanel = function (config)
{
	this._workspaceName = config.workspaceName;
	this._jcrView = config.jcrView;
	
	config.listeners = {'afterrender': {fn: this._fixOverflow, scope: this}}
	
	org.ametys.repository.PropertiesFormPanel.superclass.constructor.call(this, config);
}

Ext.extend(org.ametys.repository.PropertiesFormPanel, Ext.Panel, {
	
	cls:'properties-form-panel',
	collapsible: true,
	titleCollapse: true,
	animCollapse: true,
	hideCollapseTool: true,
	title: "<i18n:text i18n:key="PROPERTIES_PANEL_TITLE"/>",
	hideCollapseTool: true,
	floatable: false,
	
	region: 'center',
	border: false,
	forceLayout: true,
	layout: 'table',
	
	//split: true,
	autoScroll:false,
	width: 'auto',
	height: 'auto',
    
    layoutConfig: {
    	columns: 4
	},
	labelAlign: 'left',
	labelWidth: 200,
	
	fileUpload: true
});

org.ametys.repository.PropertiesFormPanel.prototype._fixOverflow = function ()
{
	this.bwrap.dom.style.overflow = 'visible';
	this.bwrap.child('.x-panel-body').dom.style.overflow = 'visible';
}

org.ametys.repository.PropertiesFormPanel.prototype._workspaceName;
org.ametys.repository.PropertiesFormPanel.prototype._node;
org.ametys.repository.PropertiesFormPanel.prototype._jcrView;
org.ametys.repository.PropertiesFormPanel.prototype.setNode = function ()
{
	this._node = node;
}

org.ametys.repository.PropertiesFormPanel.prototype.updatePanel = function (workspaceName, node, properties)
{
	this._node = node;
	this._workspaceName = workspaceName;
	
	this._drawProperties (properties);
}

Ext.override(Ext.form.TriggerField, {
	actionMode: 'wrap',
	onShow: Ext.form.TriggerField.superclass.onShow,
	onHide: Ext.form.TriggerField.superclass.onHide
});

org.ametys.repository.PropertiesFormPanel.prototype._removeOldProperties = function ()
{
	this.removeAll ();
	this.layout.cells = [];
	this.layout.currentRow = 0;
	this.layout.currentColumn = 0;
	
	var tbody = this.el.child('div.x-panel-bwrap div.x-panel-body table.x-table-layout tbody').dom;
	while (tbody.firstChild != null)
	{
		tbody.removeChild(tbody.firstChild);
	}
	this.doLayout();
}
org.ametys.repository.PropertiesFormPanel.prototype._drawProperties = function (properties)
{
	this._removeOldProperties();
	
	for (var i=0; i &lt; properties.length; i++)
	{
		var property = properties[i];
		
		// get property attributes and value(s)
		var name = property.getAttribute("name");
		var type = property.getAttribute("type");
		var multiple = property.getAttribute("multiple") == 'true';
		
		var editable = property.getAttribute("editable") == 'true';

		var values = [];

		if (multiple)
		{
			var nodeValues = property.selectNodes("value");
			for (var j=0 ; j &lt; nodeValues.length ; j++)
			{
				values[j] = nodeValues[j][org.ametys.servercomm.ServerComm.xmlTextContent];
			}
		}
		else 
		{
			values[0] = property.selectSingleNode("value")[org.ametys.servercomm.ServerComm.xmlTextContent];
		}
		
		this._drawProperty (name, type, values, multiple, editable);
		this.doLayout();
		
	}
	//end for each property
}

org.ametys.repository.PropertiesFormPanel.prototype._deleteProperty = function (name)
{
	Ext.Msg.confirm("<i18n:text i18n:key="DELETE_BUTTON_TEXT"/>", 
			"<i18n:text i18n:key="DELETE_PROPERTY_CONFIRM"/> '" + name + "' ?", 
			function (answer) {
				this._doDelete (answer, name);
			},
			this
	);
}

org.ametys.repository.PropertiesFormPanel.prototype._doDelete = function (answer, name)
{
	if (answer == 'yes')
	{
		org.ametys.repository.actions.RemoveProperty (this._node, this._workspaceName, name, this._deletePropertyCb.createDelegate(this));
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._deletePropertyCb = function (node, name)
{
	org.ametys.repository.JCRViewTab._updateNodeProperties (this._jcrView, node);
}

org.ametys.repository.PropertiesFormPanel.prototype._editProperty = function (name)
{
	var form = Ext.getCmp (name + '_form');
	var valueCt = Ext.getCmp (name + '_value');
	
	if (form != null)
	{
		form.show();
		valueCt.hide();
		var inputFd = Ext.getCmp (name + '_input') || Ext.getCmp (name + '_input_0');
		inputFd.focus ();
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._saveProperty = function (field, name, value)
{
	Ext.getCmp (name + '_form').getForm().submit({
		url : contextPath + workspaceURI + '/repository/set-property',
		params: {
					workspace: this._workspaceName,
					path: this._node.attributes.path.substring(1),
					name: name,
					value: value,
					type: field.type
				},
				
		success: this._savePropertySuccess.createDelegate (this, [field, name, value]),
		failure: this._savePropertyFail.createDelegate (this)
	});
}

org.ametys.repository.PropertiesFormPanel.prototype._savePropertySuccess = function (field, name, value)
{
	var form = Ext.getCmp (name + '_form');
	var valueCt = Ext.getCmp (name + '_value');
	var inputFd = Ext.getCmp (name + '_input');
	
	var html = "";
	switch (inputFd.type)
		{
		case 'Date':
			html = value.format("<i18n:text i18n:key="DATE_HOUR_FORMAT"/>");
			break;
		case 'Reference':
			html = this._getReferenceFormatValue (typeof (value) == 'string' ? value.split(/\r?\n/g) : ['' + value], inputFd.multiple);
			break;
		case 'Binary':
			html = this._getBinaryFormatValue ([value], inputFd.multiple);
			break;
		default:
			html = this._getStringFormatValue (typeof (value) == 'string' ? value.split(/\r?\n/g) : ['' + value], inputFd.multiple);
			break;
	}
	valueCt.update(html);
	valueCt.value = typeof (value) == 'string' ? value.split(/\r?\n/g) : value;
	
	form.hide();
	valueCt.show ();
}

org.ametys.repository.PropertiesFormPanel.prototype._savePropertyFail = function (form, action)
{
	var msg = action.result.msg;
	
	new org.ametys.msg.ErrorDialog(
			"<i18n:text i18n:key="PROPERTIES_SAVE_PROPERTY_ERROR_TITLE"/>", 
			"<i18n:text i18n:key="PROPERTIES_SAVE_PROPERTY_ERROR"/>", 
			msg,
			null
	);
}

org.ametys.repository.PropertiesFormPanel.prototype._onKeyPress = function (fd, event)
{
	if (event.getKey() == event.ENTER &amp;&amp; !fd.multiple)
	{
		this._validEdit (fd);
	}
	else if (event.getKey() == event.ESC)
	{
		this._cancelEdit (fd);
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._validEdit = function (fd)
{
	var name = fd.getName();
	var value = fd.fileInput != null ? fd.fileInput.getValue() : fd.getValue();
	
	var form = Ext.getCmp (name + '_form');
	var valueCt = Ext.getCmp (name + '_value');
	if (valueCt.value == value)
	{
		form.hide();
		valueCt.show ();
	}
	else
	{
		this._saveProperty (fd, name, value);
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._cancelEdit = function (fd)
{
	var name = fd.getName();
	var valueCt = Ext.getCmp (name + '_value');
	var form = Ext.getCmp (name + '_form');
	fd.setValue (valueCt.value);
	
	form.hide();
	valueCt.show ();
}

org.ametys.repository.PropertiesFormPanel.prototype._onDateKeyPress = function (fd, event)
{
	if (event.getKey() == event.ENTER)
	{
		var name = fd.getName();
		
		var dateFd = Ext.getCmp(name + '_date');
		var hourFd = Ext.getCmp(name + '_hour');
		
		var hour = hourFd.getRawValue();
		var stringDate = dateFd.getValue().format('Y-m-d ') + hour;
		var value = Date.parseDate(stringDate, 'Y-m-d H:i:s');
		
		this._saveProperty (Ext.getCmp(name + '_input'), name, value);
	}
	else if (event.getKey() == event.ESC)
	{
		var name = fd.getName();
		
		var valueCt = Ext.getCmp (name + '_value');
		
		var date = Date.parseDate(valueCt.value, 'Y-m-d\\TH:i:s');
		Ext.getCmp (name + '_date').setValue(date);
		Ext.getCmp (name + '_hour').setValue(date);
		
		Ext.getCmp (name + '_form').hide();
		valueCt.show ();
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._goToReference  = function (uuid)
{
	var serverMessage = new org.ametys.servercomm.ServerMessage(null, workspaceURI + '/repository/path-by-uuid',
			{
				workspace: this._workspaceName,
				uuid: uuid
			}, 
			org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS,  
			null, 
			this, 
			null);
	
	var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
	
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="ERROR_MSG_SOLVE_UUID"/>", response, "org.ametys.repository.actions.SolveUUID"))
	{
		return;
	}
	
	var path = response.selectSingleNode("path")[org.ametys.servercomm.ServerComm.xmlTextContent];
	this._node.getOwnerTree().selectNodeByPath (path);
}

/**
 * Read a binary from a binary property (using servercomm)
 * FIXME : binary received in XML response from servercomm, bug if not XML binary (eg picture)
 */
org.ametys.repository.PropertiesFormPanel.prototype._downloadBinary = function (property)
{
	var path = this._node.attributes.path.substring(1);
	
	var args = 'workspace=' + this._workspaceName + '&amp;path=' + encodeURIComponent(path) + '&amp;property=' + property;
	window.location.href = context.workspaceContext + '/download?' + args;
	return;
    
    window.open(context.workspaceContext + '/repository/read-binary?workspace=' + this._workspaceName + '&amp;path=' + path + '&amp;mime=' + mimeType + '&amp;property=' + property);
}

org.ametys.repository.PropertiesFormPanel.prototype._downloadBinaryMetadata = function(mimeType, property)
{
	var path = this._node.attributes.path.substring(1);
	return contextPath + workspaceURI + '/binary/' + this._workspaceName + path + '/' + property + '?mime-type=' + mimeType;
}

org.ametys.repository.PropertiesFormPanel.prototype._drawProperty = function (name, type, values, multiple, editable)
{
	// delete button
	var btn = new Ext.Button({
		width: 30,
		cls: 'property-btn',
		icon: context.workspaceContext + '/resources/img/delete13.png',
		tooltip: "<i18n:text i18n:key="DELETE_BUTTON_TEXT"/>",
		scope: this,
		handler: function (btn, event) { this._deleteProperty(name)},
		hidden: !editable
	});
	this.add (btn);
	
	// edit button
	var btn = new Ext.Button({
		width: 30,
		cls: 'property-btn',
		icon: context.workspaceContext + '/resources/img/edit_13.png',
		tooltip: "<i18n:text i18n:key="EDITION_BUTTON_TEXT"/>",
		scope: this,
		handler: function (btn, event) { this._editProperty(name)},
		hidden: !editable
	});
	this.add (btn);
	
	var label = "&lt;b&gt;" + name + "&lt;/b&gt; (" + type + ")"
	
	// label
	var labelCt = new org.ametys.HtmlContainer ({
		cls: 'property-label',
		width: 200,
		html: label
	});
	this.add (labelCt);
	
	// value (read mode)
	
	var html = '';
	switch (type)
	{
		case 'Date':
			html = this._getDateFormatValue (values, multiple);
			break;
		case 'Reference':
			html = this._getReferenceFormatValue (values, multiple);
			break;
		case 'Binary':
			html = this._getBinaryFormatValue (values, multiple);
			break;
		default:
			html = this._getStringFormatValue (values, multiple);
			break;
	}
	
	var val = ""
	for (var i=0; i &lt; values.length; i++)
	{
		if (i != 0)
			val += "\n";
		val += values[i];
	}
	
	var valueCt = new org.ametys.HtmlContainer ({
		id: name + '_value',
		cls: 'property-value' + (!editable ? '-disabled': ''),
		html: html,
		value: multiple ? val : values[0]
	});
	
	var form =  new Ext.form.FormPanel ({
    	id: name + '_form',
    	border: false,
    	fileUpload: true,
		hidden: true
    });
    
	var propertyCt = new Ext.Panel ({
		border: false,
		items:[ valueCt,
		        form
		 ]      
	});
	
	this.add (propertyCt);
	this.doLayout();
	
	if (type == 'Reference')
	{
		valueCt.getEl().addListener("click", function () { this._goToReference(values[0])}, this);
	}
	else if (type == 'Binary')
	{
		valueCt.getEl().addListener("click", function () { this._downloadBinary(name)}, this);
	}
	
	if (editable)
	{
		if (type != 'Reference' &amp;&amp; type != 'Reference')
		{
			valueCt.getEl().addListener("dblclick", function () { this._editProperty(name)}, this);
		}
		
		switch (type)
		{
			case 'String':
			case 'Name':
			case 'Path':
				var items = this._drawStringInput (name, type, values, multiple);
				form.add (items);
				break;
		
			case 'Date':
				var inputFd = this._drawDateInput (name, type, values, multiple);
				form.add (inputFd);
				break;
				
			case 'Reference':
				var inputFd = this._drawStringInput (name, type, values, multiple);
				form.add (inputFd);
				break;		
				
			case 'Boolean':
				var inputFd = this._drawBooleanInput (name, type, values, multiple);
				form.add (inputFd);
				break;
				
			case 'Long':
				var inputFd = this._drawLongInput (name, type, values, multiple);
				form.add (inputFd);
				break;	
				
			case 'Double':
				var inputFd = this._drawDoubleInput (name, type, values, multiple);
				form.add (inputFd);
				break;	

			case 'Binary':	
				var inputFd = this._drawBinaryInput (name, type, values);
				form.add (inputFd);
				break;
		}
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._getStringFormatValue = function (values, multiple)
{
	if (!multiple)
	{
		return values[0];
	}
	else
	{
		var val = ""
		for (var i=0; i &lt; values.length; i++)
		{
			if (i != 0)
				val += "&lt;br/&gt;";
			val += values[i];
		}
		return val;
	}
}

org.ametys.repository.PropertiesFormPanel.prototype._drawStringInput = function (name, type, values, multiple)
{
	var items = [];
	
	if (!multiple)
	{
		var inputFd = new Ext.form.TextField({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: values,
			type: type,
			width:200,
			enableKeyEvents: true,
			multiple: multiple
		});	    
		
		inputFd.addListener ('keypress', this._onKeyPress, this);
		inputFd.addListener ('blur', this._validEdit, this);
		
		items.push(inputFd);
	}
	else
	{
		var val = ""
		for (var i=0; i &lt; values.length; i++)
		{
			if (i != 0)
			{
				val += '\n';
			}
			val += values[i].trim();
		}
		
		var inputFd = new  Ext.form.TextArea({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: val,
			type: type,
			width:200,
			height: 50,
			multiple: true,
			enableKeyEvents: true
		});	    
		inputFd.addListener ('blur', this._validEdit, this);
		inputFd.addListener ('keypress', this._onKeyPress, this);
		
		items.push (inputFd);
	}
	
	return items;
	
}

org.ametys.repository.PropertiesFormPanel.prototype._getDateFormatValue = function (values)
{
	var date = Date.parseDate(values[0], 'Y-m-d\\TH:i:s');
	return date.format("<i18n:text i18n:key="DATE_HOUR_FORMAT"/>");
}

org.ametys.repository.PropertiesFormPanel.prototype._drawDateInput= function (name, type, values)
{
	var date = Date.parseDate(values[0], 'Y-m-d\\TH:i:s');
	
	// date field
	var dateFd = new Ext.form.DateField({
		hideLabel: true,
		id: name + '_date',
		ctCls: 'date-input',
		type: type,
		name: name,
		value: date,
		format: "<i18n:text i18n:key="DATE_FIELD_FORMAT"/>",
		width: 80,
		enableKeyEvents: true,
		allowBlank:false
	});
	dateFd.addListener ('keypress', this._onDateKeyPress, this);
	
	// hour field
	var hourFd = new Ext.form.TimeField({
		hideLabel: true,
		id: name + '_hour',
		type: type,
		name: name,
		format: 'H:i:s',
		increment: 5,
		value: date,
		type: type,
		width: 75,
		enableKeyEvents: true,
		allowBlank:false
	});
	hourFd.addListener ('keypress', this._onDateKeyPress, this);
	
	var inputFd = new Ext.Panel({
			id: name + '_input',
			type: type,
			border: false,
			items: [
			    dateFd,
				hourFd
			]
	});

	return inputFd;
	
}

org.ametys.repository.PropertiesFormPanel.prototype._drawLongInput = function (name, type, values, multiple)
{
	var items = [];
	
	if (!multiple)
	{
		var inputFd = new  Ext.form.NumberField ({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: values[0],
			type: type,
			width:200,
			allowDecimals: false,
			enableKeyEvents: true,
			allowBlank:false
		});	    
		
		inputFd.addListener ('keypress', this._onKeyPress, this);
		inputFd.addListener ('blur', this._validEdit, this);
		
		items.push(inputFd);
	}
	else
	{
		var val = ""
		for (var i=0; i &lt; values.length; i++)
		{
			if (i != 0)
			{
				val += '\n';
			}
			val += values[i].trim();
		}
		
		var inputFd = new  Ext.form.TextArea({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: val,
			type: type,
			width:200,
			height: 50,
			multiple: true,
			enableKeyEvents: true
		});	    
		inputFd.addListener ('blur', this._validEdit, this);
		inputFd.addListener ('keypress', this._onKeyPress, this);
		
		items.push (inputFd);
	}
	
	return items;
}

org.ametys.repository.PropertiesFormPanel.prototype._drawDoubleInput = function (name, type, values, multiple)
{
var items = [];
	
	if (!multiple)
	{
		var inputFd = new Ext.form.NumberField ({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: values[0],
			type: type,
			width:200,
			allowDecimals: true,
			enableKeyEvents: true,
			allowBlank: false
		});	    
		
		inputFd.addListener ('keypress', this._onKeyPress, this);
		inputFd.addListener ('blur', this._validEdit, this);
		
		items.push(inputFd);
	}
	else
	{
		var val = ""
		for (var i=0; i &lt; values.length; i++)
		{
			if (i != 0)
			{
				val += '\n';
			}
			val += values[i].trim();
		}
		
		var inputFd = new  Ext.form.TextArea({
			hideLabel: true,
			id: name + '_input',
			name: name,
			value: val,
			type: type,
			width:200,
			height: 50,
			multiple: true,
			enableKeyEvents: true
		});	    
		inputFd.addListener ('blur', this._validEdit, this);
		inputFd.addListener ('keypress', this._onKeyPress, this);
		
		items.push (inputFd);
	}
	
	return items;
}

org.ametys.repository.PropertiesFormPanel.prototype._drawBooleanInput = function (name, type, values)
{
	var inputFd = new Ext.form.ComboBox({
		hideLabel: true,
		id: name + '_input',
		name: name,
		triggerAction: 'all',
		type: type,
		width : 70,
		store: [true, false],
		enableKeyEvents: true,
		value: values[0]
	});	    
	
	inputFd.addListener ('keypress', this._onKeyPress, this);
	inputFd.addListener ('blur', this._validEdit, this);
	
	return inputFd;
}

org.ametys.repository.PropertiesFormPanel.prototype._getReferenceFormatValue = function (values)
{
	return values != null &amp;&amp; values.length != 0 ? '&lt;a href="#"&gt;' + values[0] + '&lt;/a&gt;' : '';
}

org.ametys.repository.PropertiesFormPanel.prototype._getBinaryFormatValue = function (values)
{
	return '&lt;a href="#"&gt;<i18n:text i18n:key='DOWNLOAD_BUTTON_TEXT'/>&lt;/a&gt;';
}

org.ametys.repository.PropertiesFormPanel.prototype._drawBinaryInput = function (name, type, values, mimeType, editable)
{
	var inputFd = new Ext.ux.form.FileUploadField({
		hideLabel: true,
		id: name + '_input',
		name: name,
		value: values[0],
		width: 200,
		type: type,
		enableKeyEvents: true
	});
	inputFd.addListener ('keypress', this._onKeyPress, this);
	inputFd.addListener ('afterrender', function () {this.container.child('div').child('input.x-form-text').setWidth(140);}, inputFd);
	
	return inputFd;
}

