/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.repository.tree');

org.ametys.repository.tree.JCRTreePanel = function(config) 
{
    config.loader = new org.ametys.repository.RepositoryXmlLoader ({
        dataUrl: context.workspaceContext + '/repository/nodes',
        listeners: {'beforeload': {fn: this._onBeforeLoad, scope: this}},
        timeout: 180000
    });	
	
    config.root = new Ext.tree.AsyncTreeNode({
		draggable: false,
		icon: context.workspaceContext + '/resources/img/jcr_node_16.png',
        text: "<i18n:text i18n:key="WORKSPACE_REPOSITORY_JCR_ROOT_NODE_TEXT"/>",
        id: '/',
        path: ''
    })
    
	org.ametys.repository.tree.JCRTreePanel.superclass.constructor.call(this, config);
	
	this.addListener('resize', this._onResize, this);
}; 

Ext.extend(org.ametys.repository.tree.JCRTreePanel, Ext.tree.TreePanel, {
	autoScroll: false,
    animate: true,
    enableDD: false,
    containerScroll: true,

    /**
     * @cfg {String} defaultWorkspaceName The default workspace. Defaults to 'default'
     */
	defaultWorkspaceName : 'default'
});

org.ametys.repository.tree.JCRTreePanel.prototype._onResize = function()
{
	this._ulWrapper.setWidth(this.body.getWidth());
	this._ulWrapper.setHeight(this.body.getHeight() - (this._form != null ? this._form.getHeight() : 0));
}

org.ametys.repository.tree.JCRTreePanel.prototype.onRender = function(ct, position)
{
	org.ametys.repository.tree.JCRTreePanel.superclass.onRender.call(this, ct, position);

	this._form = new Ext.Panel ({
		layout: 'column',
		border: false,
		cls: 'jcr-tree-panel-form',
		labelWidth :80,
		labelSeparator: ''
	});
	
	this._workspaceCombo = this._drawWorkspaceCombo ();
	this._workspaceCombo.columnWidth = 1;
	
	this._form.add (new Ext.form.Label({
		width: 100, 
		bodyStyle: { 'textAlign': 'center'},
    	text: "<i18n:text i18n:key="WORKSPACE_REPOSITORY_WORKSPACE_NAME"/>"
    	})
	);
	this._form.add (this._workspaceCombo);
	
	this.add(this._form);
	
	this._ulWrapper = this.body.child("ul").wrap({ style: 'overflow: auto;' });

	this._loadWorkspaces ();
}

/**
 * This function creates a root node from the current site name and language
 * @return The created root node
 */
org.ametys.repository.tree.JCRTreePanel.prototype.createRootNode = function ()
{
	var rootNode = new Ext.tree.AsyncTreeNode({
		draggable: false,
		icon: getWorkspaceResources() + '/img/jcr_node_16.png',
        text: "<i18n:text i18n:key="WORKSPACE_REPOSITORY_JCR_ROOT_NODE_TEXT"/>",
        id: '/',
        path: '/'
    })
	this.setRootNode(rootNode);	
}


org.ametys.repository.tree.JCRTreePanel.prototype._drawWorkspaceCombo = function ()
{
	var store = new Ext.data.SimpleStore({
        fields: ['name'],
        data: []
    });
	
	var combo = new Ext.form.ComboBox({
		width: 160,
		minListWidth: 120,
		
    	forceSelection: true,
    	editable: false,
    	mode: 'local',
    	resizable: false,
    	triggerAction: 'all',
    	 
    	fieldLabel: "<i18n:text i18n:key="WORKSPACE_REPOSITORY_WORKSPACE_NAME"/>",
    	
    	store: store,
    	
    	name: 'name',
        valueField: 'name',
        displayField: 'name',
        
        tpl: new Ext.XTemplate(
        		'&lt;tpl for="."&gt;',
    			'&lt;div class="x-combo-list-item combo-list-workspace-item"&gt;',
            		'{name}',
            		'&lt;div class="x-clear"&gt;&lt;/div&gt;',
            	'&lt;/div&gt;',
            	'&lt;/tpl&gt;')
    });
	
	combo.addListener ('select', this._workspaceChange, this);
	
	return combo;
}

org.ametys.repository.tree.JCRTreePanel.prototype._workspaceChange = function (combo, record, index)
{
	if (this._workspaceName == record.data.name)
		return;
	
	this._workspaceName = record.data.name;
	
	var node = this.getSelectionModel().getSelectedNode();
    var currentPath = node != null ? node.attributes.path : null;

    this._loadTree(this.selectNodeByPath.createDelegate(this, [currentPath]));
}

org.ametys.repository.tree.JCRTreePanel.prototype.selectNodeByPath = function (path)
{
	var rootText = this.getRootNode().text;
	this.expandPath("/" + rootText + path, 'text', this._expandPathCb);
}
org.ametys.repository.tree.JCRTreePanel.prototype._expandPathCb = function(bSuccess, oLastNode)
{
	if (oLastNode != null)
	{
		oLastNode.ensureVisible(); // scroll to node
		oLastNode.select(); // select node
	}
}

org.ametys.repository.tree.JCRTreePanel.prototype._loadWorkspaces = function ()
{
	if (!this._workspaceName)
		this._workspaceName = this.defaultWorkspaceName;
	
	var serverMessage = new org.ametys.servercomm.ServerMessage(null, '/_repository/repository/workspaces.xml', {}, org.ametys.servercomm.ServerComm.PRIORITY_MAJOR, this._loadWorkspacesCb, this, []);
	org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
}
org.ametys.repository.tree.JCRTreePanel.prototype._loadWorkspacesCb = function (response, args)
{
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="WORKSPACE_REPOSITORY_WORKSPACE_NAME"/>", response, "org.ametys.repository.tree.JCRTreePanel"))
	{
		return;
	}
	
	var store = this._workspaceCombo.getStore();
	store.removeAll();	
	
	var workspaces = response.selectNodes("workspaces/workspace");
	for (var i=0; i &lt; workspaces.length; i++)
	{
		var data = {
	            name: workspaces[i][org.ametys.servercomm.ServerComm.xmlTextContent]
	    };
		store.add (new store.recordType(data));
	}
	
	// Initialize with default value
	this._workspaceCombo.setValue(this.defaultWorkspaceName);
	this._workspaceName = this.defaultWorkspaceName;
	store.sort('name', 'ASC');
	
	// Load the tree
	this._loadTree();
}

org.ametys.repository.tree.JCRTreePanel.prototype._onBeforeLoad = function (loader, node, cb)
{
    loader.baseParams = {};
    
	// set the current workspace name
	loader.baseParams['workspaceName'] = this._workspaceName;
	
    var path = node.id.substring(1, node.id.length - 1);
    loader.baseParams['path'] = node.attributes.path.substring(1);
    
    if (node.attributes.start &amp;&amp; node.attributes.end)
	{
	    loader.baseParams.start = node.attributes.start;
	    loader.baseParams.end = node.attributes.end;
	}
    
    var jcrNode = node;
    while (jcrNode.attributes.tagName == 'group')
    {
        jcrNode = jcrNode.parentNode;
    }
    
    if (jcrNode.attributes.order)
    {
        loader.baseParams.order = jcrNode.attributes.order;
    }
}

org.ametys.repository.tree.JCRTreePanel.prototype._loadTree = function (callback)
{
	this.getRootNode().reload (this._loadTreeCb.createDelegate(this, [callback]), this);
}

org.ametys.repository.tree.JCRTreePanel.prototype._loadTreeCb = function (callback)
{
	if (typeof callback == 'function')
	{
		callback ();
	}
	else
	{
		this.getSelectionModel().clearSelections();
		this.getRootNode().select();
	}
}
