/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.repository.tree');

org.ametys.repository.tree.SearchTreePanel = function(config) 
{
	config.loader = new org.ametys.repository.tree.ResultXmlLoader ({
        dataUrl: config.loaderUrl,
        listeners: {'beforeload': {fn: this._onBeforeLoad, scope: this},
					'load': {fn: this._onLoad, scope: this}
		},
		baseAttrs: {icon: config.iconNodeUrl}
    });	
	
    config.root = new Ext.tree.AsyncTreeNode({
		draggable: false,
		icon: context.workspaceContext + '/resources/img/search_16.png',
        text: "&lt;b&gt;<i18n:text i18n:key="SEARCH_ROOT_NODE_LABEL"/>&lt;/b&gt;",
        listeners: {'expand': {fn: this._onExpand, scope: this},
					'collapse': {fn: this._onCollapse, scope: this}
    	}
    })
    
    org.ametys.repository.tree.SearchTreePanel.superclass.constructor.call(this, config);
}; 

Ext.extend(org.ametys.repository.tree.SearchTreePanel, Ext.tree.TreePanel, {
	autoScroll: true,
    animate: true,
    enableDD: false,
    containerScroll: true,
    hidden: true,
    
    /**
     * @cfg {String} defaultWorkspaceName The default workspace. Defaults to 'default'
     */
	defaultWorkspaceName : 'default'
});

org.ametys.repository.tree.SearchTreePanel.prototype.onRender = function(ct, position)
{
	org.ametys.repository.tree.SearchTreePanel.superclass.onRender.call(this, ct, position);
	
	this._workspaceName = this.defaultWorkspaceName;
}

org.ametys.repository.tree.SearchTreePanel.prototype._xpathQuery;
org.ametys.repository.tree.SearchTreePanel.prototype.setXPathQuery = function (xpathQuery, language)
{
	this._xpathQuery = xpathQuery;
	this._language = language != null ? language : 'xpath';
}

org.ametys.repository.tree.SearchTreePanel.prototype._onBeforeLoad = function (loader, node, cb)
{
	if (this._waitMsg != null)
	{
		this._waitMsg.hide();
	}
	this._waitMsg = new org.ametys.msg.Mask(this.ownerCt.getEl(), "<i18n:text i18n:key="XPATH_QUERY_EXECUTE_WAITING_MSG"/>");
	
	// set the current workspace name
	loader.baseParams['workspaceName'] = this._workspaceName;
	loader.baseParams['query'] = this._xpathQuery;
	loader.baseParams['language'] = this._language;
}

org.ametys.repository.tree.SearchTreePanel.prototype._onLoad = function (loader, node, response)
{
	if (response == null || response.status != 200)
	{
		if (this._waitMsg != null)
		{
			this._waitMsg.hide();
			delete this._waitMsg;
		}
		
		Ext.Msg.show({
			title: "<i18n:text i18n:key="XPATH_QUERY_PROMPT_TITLE"/>",
			msg: "<i18n:text i18n:key="XPATH_QUERY_EXECUTE_UNEXCEPTED_ERROR_MSG"/>",
			buttons: Ext.Msg.OK,
			icon: Ext.MessageBox.ERROR
		});
		
		node.setText("&lt;b&gt;<i18n:text i18n:key="SEARCH_ROOT_NODE_LABEL"/>" + ' : 0 <i18n:text i18n:key="SEARCH_NB_RESULTS"/> - ' + '&lt;span style="color:red"&gt;<i18n:text i18n:key="XPATH_QUERY_EXECUTE_ERROR_NODE"/>&lt;/span&gt;' + '&lt;/b&gt;');
		node.removeAll();
		
		if (!this.isVisible())
		{
			this.setVisible(true);
			this.ownerCt.doLayout();
		}
		return;
	}
	
	var nbResults = node.childNodes.length;
	
	if (nbResults == 1 &amp;&amp; node.childNodes[0].attributes.tagName == 'error')
	{
		if (this._waitMsg != null)
		{
			this._waitMsg.hide();
			delete this._waitMsg;
		}
		
		Ext.Msg.show({
			title: "<i18n:text i18n:key="XPATH_QUERY_PROMPT_TITLE"/>",
			msg: "<i18n:text i18n:key="XPATH_QUERY_EXECUTE_ERROR_MSG"/>",
			buttons: Ext.Msg.OK,
			icon: Ext.MessageBox.ERROR
		});
		
		
		node.setText("&lt;b&gt;<i18n:text i18n:key="SEARCH_ROOT_NODE_LABEL"/>" + ' : 0 <i18n:text i18n:key="SEARCH_NB_RESULTS"/> - ' + '&lt;span style="color:red"&gt;<i18n:text i18n:key="XPATH_QUERY_EXECUTE_ERROR_NODE"/>&lt;/span&gt;' + '&lt;/b&gt;');
		node.removeAll();
		
		if (!this.isVisible())
		{
			this.setVisible(true);
			this.ownerCt.doLayout();
		}
		return;
	}
	
	node.setText("&lt;b&gt;<i18n:text i18n:key="SEARCH_ROOT_NODE_LABEL"/>" + ' : ' + nbResults + " <i18n:text i18n:key="SEARCH_NB_RESULTS"/>" + '&lt;/b&gt;');
	
	if (!this.isVisible())
	{
		this.setVisible(true);
		this.ownerCt.doLayout();
	}
	
	if (this._waitMsg != null)
	{
		this._waitMsg.hide();
		delete this._waitMsg;
	}
	
	this.expand();
	this.doLayout();
}

org.ametys.repository.tree.SearchTreePanel.prototype._onExpand = function (node)
{
	this.setHeight(200);
	this.ownerCt.doLayout();
}

org.ametys.repository.tree.SearchTreePanel.prototype._onCollapse = function (node)
{
	this.setHeight(20);
	this.ownerCt.doLayout();
}
